@@ -24,9 +24,13 @@ except ImportError:
 # Import ioflo libs
 from ioflo.base.odicting import odict
 from ioflo.base import aiding
+from ioflo.base import nonblocking
 from ioflo.base import storing
 
+# Import raet libs
+from ..abiding import *  # import globals
 from .. import raeting
+from ..raeting import PcktKind, TrnsKind, CoatKind, FootKind, BodyKind, HeadKind
 from .. import nacling
 from .. import stacking
 from . import keeping
@@ -78,12 +82,12 @@ class RoadStack(stacking.KeepStack):
         The local estate role identifier for key management
     '''
     Count = 0 # count of Stack instances to give unique stack names
-    Hk = raeting.headKinds.raet # stack default
-    Bk = raeting.bodyKinds.json # stack default
-    Fk = raeting.footKinds.nacl # stack default
-    Ck = raeting.coatKinds.nacl # stack default
+    Hk = HeadKind.raet.value # stack default
+    Bk = BodyKind.json.value # stack default
+    Fk = FootKind.nacl.value # stack default
+    Ck = CoatKind.nacl.value # stack default
     Bf = False # stack default for bcstflag
-    Wf = False # stack default for waitflag
+    BurstSize = 0  # stack default for max segments in each burst, 0 = no limit
     Period = 1.0 # stack default for keep alive
     Offset = 0.5 # stack default for keep alive
     Interim = 3600 # stack default for reap timeout
@@ -153,7 +157,7 @@ class RoadStack(stacking.KeepStack):
                                         **kwa)
         self.kind = kind # application kind associated with the local estate
         self.mutable = mutable # road data mutability
-        self.joinees = odict() # remotes associated with vacuous joins, keyed by ha
+        self.joinees = odict() # remotes for vacuous joins, keyed by ha
         self.alloweds = odict() # allowed remotes keyed by name
         self.aliveds =  odict() # alived remotes keyed by name
         self.reapeds =  odict() # reaped remotes keyed by name
@@ -184,7 +188,7 @@ class RoadStack(stacking.KeepStack):
         '''
         Create local listening server for stack
         '''
-        server = aiding.SocketUdpNb(ha=self.ha,
+        server = nonblocking.SocketUdpNb(ha=self.ha,
                         bufsize=raeting.UDP_MAX_PACKET_SIZE * self.bufcnt)
         return server
 
@@ -441,16 +445,26 @@ class RoadStack(stacking.KeepStack):
         Process packet via associated transaction or
         reply with new correspondent transaction
         '''
-        console.verbose("{0} received packet data\n{1}\n".format(self.name, packet.data))
-        console.verbose("{0} packet packet index = '{1}'\n".format(self.name, packet.index))
+        console.profuse("{0} received packet data\n{1}\n".format(self.name, packet.data))
+        console.verbose("{0} received packet index: (rf={1[0]}, le={1[1]}, re={1[2]},"
+                " si={1[3]}, ti={1[4]}, bf={1[5]})\n".format(self.name, packet.index))
+        try:
+            tkname = TrnsKind(packet.data['tk'])
+        except ValueError as ex:
+            tkname = None
+        try:
+            pkname = PcktKind(packet.data['pk'])
+        except ValueError as ex:
+            pkname = None
+        console.verbose("{0} received trans kind = '{1}' packet kind = '{2}'"
+                        "\n".format(self.name, tkname, pkname))
 
         bf = packet.data['bf']
         if bf:
             return  # broadcast transaction not yet supported
 
-        nuid = packet.data['de']
-
-        fuid = packet.data['se']
+        de = packet.data['de']  # remote nuid
+        se = packet.data['se']  # remote fuid
         tk = packet.data['tk']
         pk = packet.data['pk']
         cf = packet.data['cf']
@@ -458,77 +472,79 @@ class RoadStack(stacking.KeepStack):
 
         remote = None
 
-        if tk in [raeting.trnsKinds.join]: # join transaction
-            rha = (packet.data['sh'],  packet.data['sp'])
+        if tk in [TrnsKind.join]: # join transaction
+            sha = (packet.data['sh'],  packet.data['sp'])
             if rsid != 0: # join  must use sid == 0
                 emsg = ("Stack '{0}'. Nonzero join sid '{1}' in packet from {2}."
-                       " Dropping...\n".format(self.name, rsid, rha))
+                       " Dropping...\n".format(self.name, rsid, sha))
                 console.terse(emsg)
                 self.incStat('join_invalid_sid')
                 return
 
-            if cf: # packet source is joinent, destination (self) is joiner
-                if nuid == 0: # invalid join
+            if cf: # cf = not rf, packet source is joinent, destination (self) is joiner
+                if de == 0: # invalid since joiner rxed packet de (nuid) == 0
                     emsg = ("Stack '{0}'. Invalid join correspondence from '{1}',"
-                            " nuid zero . Dropping...\n".format(self.name, rha))
+                            " nuid zero . Dropping...\n".format(self.name, sha))
                     console.terse(emsg)
                     self.incStat('join_invalid_nuid')
                     return
-                if fuid == 0: # vacuous join match remote by rha from .joinees
-                    remote = self.joinees.get(rha, None)
-                    if remote and remote.nuid != nuid: # prior different
+                if se == 0: # vacuous join since se (fuid) == 0
+                    remote = self.joinees.get(sha, None)  # match remote by rha from .joinees
+                    if remote and remote.nuid != de: # prior different
                         emsg = ("Stack '{0}'. Invalid join correspondence from '{1}',"
                             "nuid  {2} mismatch prior {3} . Dropping...\n".format(
-                                self.name, rha, nuid, remote.nuid))
+                                self.name, sha, de, remote.nuid))
                         self.incStat('join_mismatch_nuid') # drop inconsistent nuid
                         return
                 else: # non vacuous join match by nuid from .remotes
-                    remote = self.remotes.get(nuid, None)
+                    remote = self.remotes.get(de, None)
 
-            else: # (not cf) # source is joiner, destination (self) is joinent
-                if fuid == 0: # invalid join
+            else: # (rf = not cf) # source is joiner, destination (self) is joinent
+                if se == 0: # invalid join
                     emsg = ("Stack '{0}'. Invalid join initiatance from '{1}',"
-                            "fuid zero. Dropping...\n".format(self.name, rha))
+                            "fuid zero. Dropping...\n".format(self.name, sha))
                     console.terse(emsg)
                     self.incStat('join_invalid_fuid')
                     return
-                if nuid == 0: # vacuous join match remote by rha from joinees
-                    remote = self.joinees.get(rha, None)
-                    if remote and remote.fuid != fuid: # check if prior is stale
-                        emsg = ("Stack '{0}'. Prior stale join initiatance from '{1}',"
-                            " fuid {2} mismatch prior {3}. Removing prior...\n".format(
-                                    self.name, rha, fuid, remote.fuid))
-                        console.terse(emsg)
-                        del self.joinees[rha] # remove prior stale vacuous joinee
-                        remote = None # reset
+                if de == 0: # vacuous join match remote by rha from joinees
+                    remote = self.joinees.get(sha, None)
+                    if remote and remote.fuid != se: # check if prior is stale
+                        if remote.fuid != 0:  # stale
+                            emsg = ("Stack '{0}'. Prior stale join initiatance from '{1}',"
+                                " fuid {2} mismatch prior {3}. Removing prior...\n".format(
+                                        self.name, sha, se, remote.fuid))
+                            console.terse(emsg)
+                            del self.joinees[sha] # remove prior stale vacuous joinee
+                            remote = None # reset
 
-                    if not remote: # no current joinees for intiator at rha
+                    if not remote: # no current joinees for remote initiator at rha
                         # is it not first packet of join
-                        if pk not in [raeting.pcktKinds.request]:
-                            emsg = ("Stack '{0}'. Invalid join initiatance from '{1}',"
+                        if pk not in [PcktKind.request]:
+                            emsg = ("Stack '{0}'. Stale join initiatance from '{1}',"
                                 " Not a request and no remote. Dropping...\n".format(
-                                        self.name, rha))
+                                        self.name, sha))
+                            console.terse(emsg)
                             self.incStat('join_stale')
                             return
 
-                        # create vacuous remote and assign to joinees
+                        # create vacuous remote will be assigned to joinees in joinent
                         remote = estating.RemoteEstate(stack=self,
-                                                       fuid=fuid,
+                                                       fuid=0,  # was fuid=se
                                                        sid=rsid,
-                                                       ha=rha)
-                        # in joinent if vacuous self.joinees[rha] = remote
+                                                       ha=sha)
+
                 else: # nonvacuous join match by nuid from .remotes
-                    remote = self.remotes.get(nuid, None)
+                    remote = self.remotes.get(de, None)
                     if not remote: # remote with nuid not exist
                         # reject renew , so tell it to retry with vacuous join
                         emsg = ("Stack '{0}'. Stale nuid '{1}' in packet from {2}."
-                                " Renewing....\n".format( self.name, nuid, rha))
+                                " Renewing....\n".format( self.name, de, sha))
                         console.terse(emsg)
                         self.incStat('stale_nuid')
                         remote = estating.RemoteEstate(stack=self,
-                                                        fuid=fuid,
+                                                        fuid=se,
                                                         sid=rsid,
-                                                        ha=rha)
+                                                        ha=sha)
                         self.replyStale(packet, remote, renew=True) # nack stale transaction
                         return
 
@@ -540,15 +556,15 @@ class RoadStack(stacking.KeepStack):
                 self.incStat('invalid_sid')
                 return
 
-            if nuid == 0 or fuid == 0:
+            if de == 0 or se == 0:
                 emsg = ("Stack '{0}'. Invalid nonjoin from remote '{1}'."
                         " Zero nuid {2} or fuid {3}. Dropping...\n".format(
-                            self.name, rha, nuid, fuid))
+                            self.name, sha, de, se))
                 console.terse(emsg)
                 self.incStat('invalid_uid')
                 return
 
-            remote = self.remotes.get(nuid, None)
+            remote = self.remotes.get(de, None)
 
             if remote:
                 if not cf: # packet from remotely initiated transaction
@@ -574,13 +590,13 @@ class RoadStack(stacking.KeepStack):
                 trans.receive(packet)
                 return
 
-        if cf: #packet from correspondent to non-existent locally initiated transaction
+        if cf:  # cf = not rf packet from correspondent to non-existent locally initiated transaction
             self.stale(packet)
             return
 
         if not remote:
             emsg = ("Stack '{0}'. Unknown remote destination '{1}'. "
-                    "Dropping...\n".format(self.name, nuid))
+                    "Dropping...\n".format(self.name, de))
             console.terse(emsg)
             self.incStat('unknown_destination_uid')
             return
@@ -591,24 +607,27 @@ class RoadStack(stacking.KeepStack):
         '''
         Create correspondent transaction remote and handle packet
         '''
-        if (packet.data['tk'] == raeting.trnsKinds.join and
-                packet.data['pk'] == raeting.pcktKinds.request):
+        if (packet.data['tk'] == TrnsKind.join and
+                packet.data['pk'] == PcktKind.request):
             self.replyJoin(packet, remote)
             return
 
-        if (packet.data['tk'] == raeting.trnsKinds.allow and
-                packet.data['pk'] == raeting.pcktKinds.hello):
+        if (packet.data['tk'] == TrnsKind.allow and
+                packet.data['pk'] == PcktKind.hello):
             self.replyAllow(packet, remote)
             return
 
-        if (packet.data['tk'] == raeting.trnsKinds.alive and
-                packet.data['pk'] == raeting.pcktKinds.request):
+        if (packet.data['tk'] == TrnsKind.alive and
+                packet.data['pk'] == PcktKind.request):
             self.replyAlive(packet, remote)
             return
 
-        if (packet.data['tk'] == raeting.trnsKinds.message and
-                packet.data['pk'] == raeting.pcktKinds.message):
-            self.replyMessage(packet, remote)
+        if (packet.data['tk'] == TrnsKind.message and
+                packet.data['pk'] == PcktKind.message):
+            if packet.data['af']:  # packet is a stale resend
+                self.replyStale(packet, remote)
+            else:
+                self.replyMessage(packet, remote)
             return
 
         self.incStat('stale_packet')
@@ -643,12 +662,12 @@ class RoadStack(stacking.KeepStack):
         Initiate stale transaction in order to nack a stale correspondent packet
         but only for preexisting remotes
         '''
-        if packet.data['pk'] in [raeting.pcktKinds.nack,
-                                         raeting.pcktKinds.unjoined,
-                                         raeting.pcktKinds.unallowed,
-                                         raeting.pcktKinds.renew,
-                                         raeting.pcktKinds.refuse,
-                                         raeting.pcktKinds.reject,]:
+        if packet.data['pk'] in [PcktKind.nack,
+                                PcktKind.unjoined,
+                                PcktKind.unallowed,
+                                PcktKind.renew,
+                                PcktKind.refuse,
+                                PcktKind.reject,]:
             return # ignore stale nacks
         create = False
         uid = packet.data['de']
@@ -673,11 +692,11 @@ class RoadStack(stacking.KeepStack):
         '''
         Correspond to stale initiated transaction
         '''
-        if packet.data['pk'] in [raeting.pcktKinds.nack,
-                                 raeting.pcktKinds.unjoined,
-                                 raeting.pcktKinds.unallowed,
-                                 raeting.pcktKinds.refuse,
-                                 raeting.pcktKinds.reject,]:
+        if packet.data['pk'] in [PcktKind.nack,
+                                 PcktKind.unjoined,
+                                 PcktKind.unallowed,
+                                 PcktKind.refuse,
+                                 PcktKind.reject,]:
             return # ignore stale nacks
         data = odict(hk=self.Hk, bk=self.Bk)
         stalent = transacting.Stalent(stack=self,
@@ -688,7 +707,7 @@ class RoadStack(stacking.KeepStack):
                                       txData=data,
                                       rxPacket=packet)
         if renew:
-            stalent.nack(kind=raeting.pcktKinds.renew) # refuse and renew
+            stalent.nack(kind=PcktKind.renew.value) # refuse and renew
         else:
             stalent.nack()
 
@@ -738,7 +757,7 @@ class RoadStack(stacking.KeepStack):
             console.terse(emsg)
             self.incStat('invalid_remote_eid')
             return
-        data = odict(hk=self.Hk, bk=raeting.bodyKinds.raw, fk=self.Fk)
+        data = odict(hk=self.Hk, bk=BodyKind.raw.value, fk=self.Fk)
         allower = transacting.Allower(stack=self,
                                       remote=remote,
                                       timeout=timeout,
@@ -750,7 +769,7 @@ class RoadStack(stacking.KeepStack):
         '''
         Correspond to new allow transaction
         '''
-        data = odict(hk=self.Hk, bk=raeting.bodyKinds.raw, fk=self.Fk)
+        data = odict(hk=self.Hk, bk=BodyKind.raw.value, fk=self.Fk)
         allowent = transacting.Allowent(stack=self,
                                         remote=remote,
                                         sid=packet.data['si'],
@@ -843,7 +862,7 @@ class RoadStack(stacking.KeepStack):
                                           timeout=timeout,
                                           txData=data,
                                           bcst=self.Bf,
-                                          wait=self.Wf)
+                                          burst=self.BurstSize)
         messenger.message(body)
 
     def replyMessage(self, packet, remote):
